
From: Michael Ellerman <michael@ellerman.id.au>

Under some strange circumstances the iseries_veth driver can leak skbs.

Fix is simply to call dev_kfree_skb() in the right place.  Fix up the comment
as well.

Signed-off-by: Michael Ellerman <michael@ellerman.id.au>
Cc: Jeff Garzik <jgarzik@pobox.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 drivers/net/iseries_veth.c |   17 +++++++++++------
 1 files changed, 11 insertions(+), 6 deletions(-)

diff -puN drivers/net/iseries_veth.c~iseries_veth-dont-leak-skbs-in-rx-path drivers/net/iseries_veth.c
--- 25/drivers/net/iseries_veth.c~iseries_veth-dont-leak-skbs-in-rx-path	2005-05-12 01:20:42.000000000 -0700
+++ 25-akpm/drivers/net/iseries_veth.c	2005-05-12 01:20:42.000000000 -0700
@@ -1264,13 +1264,18 @@ static void veth_receive(struct veth_lpa
 
 		vlan = skb->data[9];
 		dev = veth_dev[vlan];
-		if (! dev)
-			/* Some earlier versions of the driver sent
-			   broadcasts down all connections, even to
-			   lpars that weren't on the relevant vlan.
-			   So ignore packets belonging to a vlan we're
-			   not on. */
+		if (! dev) {
+			/*
+			 * Some earlier versions of the driver sent
+			 * broadcasts down all connections, even to lpars
+			 * that weren't on the relevant vlan. So ignore
+			 * packets belonging to a vlan we're not on.
+			 * We can also be here if we receive packets while
+			 * the driver is going down, because then dev is NULL.
+			 */
+			dev_kfree_skb_irq(skb);
 			continue;
+		}
 
 		port = (struct veth_port *)dev->priv;
 		dest = *((u64 *) skb->data) & 0xFFFFFFFFFFFF0000;
_
